@@ -1,10 +1,13 @@
 /*
- * AES Key Wrap Algorithm (128-bit KEK) (RFC3394)
- * One-Key CBC MAC (OMAC1) hash with AES-128
- * AES-128 CTR mode encryption
- * AES-128 EAX mode encryption/decryption
- * AES-128 CBC
- * Copyright (c) 2003-2004, Jouni Malinen <jkmaline@cc.hut.fi>
+ * AES-based functions
+ *
+ * - AES Key Wrap Algorithm (128-bit KEK) (RFC3394)
+ * - One-Key CBC MAC (OMAC1) hash with AES-128
+ * - AES-128 CTR mode encryption
+ * - AES-128 EAX mode encryption/decryption
+ * - AES-128 CBC
+ *
+ * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -53,7 +56,7 @@ typedef struct aes_key_st AES_KEY;
  * @plain: plaintext key to be wrapped, n * 64 bit
  * @cipher: wrapped key, (n + 1) * 64 bit
  */
-void aes_wrap(u8 *kek, int n, u8 *plain, u8 *cipher)
+void aes_wrap(const u8 *kek, int n, const u8 *plain, u8 *cipher)
 {
 	u8 *a, *r, b[16];
 	int i, j;
@@ -102,7 +105,7 @@ void aes_wrap(u8 *kek, int n, u8 *plain, u8 *cipher)
  * @cipher: wrapped key to be unwrapped, (n + 1) * 64 bit
  * @plain: plaintext key, n * 64 bit
  */
-int aes_unwrap(u8 *kek, int n, u8 *cipher, u8 *plain)
+int aes_unwrap(const u8 *kek, int n, const u8 *cipher, u8 *plain)
 {
 	u8 a[8], *r, b[16];
 	int i, j;
