@@ -1,8 +1,8 @@
 #ifndef AES_WRAP_H
 #define AES_WRAP_H
 
-void aes_wrap(u8 *kek, int n, u8 *plain, u8 *cipher);
-int aes_unwrap(u8 *kek, int n, u8 *cipher, u8 *plain);
+void aes_wrap(const u8 *kek, int n, const u8 *plain, u8 *cipher);
+int aes_unwrap(const u8 *kek, int n, const u8 *cipher, u8 *plain);
 void omac1_aes_128(const u8 *key, const u8 *data, size_t data_len, u8 *mac);
 void aes_128_encrypt_block(const u8 *key, const u8 *in, u8 *out);
 void aes_128_ctr_encrypt(const u8 *key, const u8 *nonce,
