@@ -61,7 +61,7 @@ static int hostapd_ioctl(void *priv, struct prism2_hostapd_param *param,
 			 int len);
 static int hostap_set_iface_flags(void *priv, int dev_up);
 
-static void handle_data(hostapd *hapd, char *buf, size_t len, u16 stype)
+static void handle_data(hostapd *hapd, u8 *buf, size_t len, u16 stype)
 {
 	struct ieee80211_hdr *hdr;
 	u16 fc, ethertype;
@@ -131,7 +131,7 @@ static void handle_data(hostapd *hapd, char *buf, size_t len, u16 stype)
 }
 
 
-static void handle_tx_callback(hostapd *hapd, char *buf, size_t len, int ok)
+static void handle_tx_callback(hostapd *hapd, u8 *buf, size_t len, int ok)
 {
 	struct ieee80211_hdr *hdr;
 	u16 fc, type, stype;
@@ -175,7 +175,7 @@ static void handle_tx_callback(hostapd *hapd, char *buf, size_t len, int ok)
 }
 
 
-static void handle_frame(hostapd *hapd, char *buf, size_t len)
+static void handle_frame(hostapd *hapd, u8 *buf, size_t len)
 {
 	struct ieee80211_hdr *hdr;
 	u16 fc, extra_len, type, stype;
@@ -488,7 +488,7 @@ static int hostap_set_encryption(void *priv, const char *alg, u8 *addr,
 		memset(param->sta_addr, 0xff, ETH_ALEN);
 	else
 		memcpy(param->sta_addr, addr, ETH_ALEN);
-	strncpy(param->u.crypt.alg, alg, HOSTAP_CRYPT_ALG_NAME_LEN);
+	strncpy((char *) param->u.crypt.alg, alg, HOSTAP_CRYPT_ALG_NAME_LEN);
 	param->u.crypt.flags = HOSTAP_CRYPT_FLAG_SET_TX_KEY;
 	param->u.crypt.idx = idx;
 	param->u.crypt.key_len = key_len;
