@@ -273,7 +273,7 @@ bsd_set_ieee8021x(void *priv, int enabled)
 	}
 	if (!conf->wpa && !conf->ieee802_1x) {
 		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_DRIVER,
-			HOSTAPD_LEVEL_WARNING, "No 802.1x or WPA enabled!");
+			HOSTAPD_LEVEL_WARNING, "No 802.1X or WPA enabled!");
 		return -1;
 	}
 	if (conf->wpa && bsd_configure_wpa(drv) != 0) {
@@ -284,7 +284,7 @@ bsd_set_ieee8021x(void *priv, int enabled)
 	if (set80211param(priv, IEEE80211_IOC_AUTHMODE,
 		(conf->wpa ?  IEEE80211_AUTH_WPA : IEEE80211_AUTH_8021X))) {
 		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_DRIVER,
-			HOSTAPD_LEVEL_WARNING, "Error enabling WPA/802.1x!");
+			HOSTAPD_LEVEL_WARNING, "Error enabling WPA/802.1X!");
 		return -1;
 	}
 	return bsd_set_iface_flags(priv, 1);
