@@ -23,6 +23,11 @@
 
 #include <include/compat.h>
 #include <net80211/ieee80211.h>
+#ifdef WME_NUM_AC
+/* Assume this is built against BSD branch of madwifi driver. */
+#define MADWIFI_BSD
+#include <net80211/_ieee80211.h>
+#endif /* WME_NUM_AC */
 #include <net80211/ieee80211_crypto.h>
 #include <net80211/ieee80211_ioctl.h>
 
@@ -58,10 +63,14 @@ struct madwifi_driver_data {
 	struct l2_packet_data *sock_recv;	/* raw packet recv socket */
 	int	ioctl_sock;			/* socket for ioctl() use */
 	int	wext_sock;			/* socket for wireless events */
+	u8	acct_mac[ETH_ALEN];
+	struct hostap_sta_driver_data acct_data;
 };
 
 static const struct driver_ops madwifi_driver_ops;
 
+static int madwifi_sta_deauth(void *priv, u8 *addr, int reason_code);
+
 static int
 set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 {
@@ -298,7 +307,7 @@ madwifi_set_ieee8021x(void *priv, int enabled)
 	}
 	if (!conf->wpa && !conf->ieee802_1x) {
 		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_DRIVER,
-			HOSTAPD_LEVEL_WARNING, "No 802.1x or WPA enabled!");
+			HOSTAPD_LEVEL_WARNING, "No 802.1X or WPA enabled!");
 		return -1;
 	}
 	if (conf->wpa && madwifi_configure_wpa(drv) != 0) {
@@ -309,7 +318,7 @@ madwifi_set_ieee8021x(void *priv, int enabled)
 	if (set80211param(priv, IEEE80211_PARAM_AUTHMODE,
 		(conf->wpa ?  IEEE80211_AUTH_WPA : IEEE80211_AUTH_8021X))) {
 		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_DRIVER,
-			HOSTAPD_LEVEL_WARNING, "Error enabling WPA/802.1x!");
+			HOSTAPD_LEVEL_WARNING, "Error enabling WPA/802.1X!");
 		return -1;
 	}
 	return madwifi_set_iface_flags(priv, 1);
@@ -468,16 +477,13 @@ madwifi_get_seqnum(void *priv, u8 *addr, int idx, u8 *seq)
 static int 
 madwifi_flush(void *priv)
 {
-#if 0
-	struct madwifi_driver_data *drv = priv;
-	struct prism2_hostapd_param param;
-
-	memset(&param, 0, sizeof(param));
-	param.cmd = PRISM2_HOSTAPD_FLUSH;
-	return madwifi_ioctl(drv, &param, sizeof(param));
-#else
+#ifdef MADWIFI_BSD
+	u8 allsta[IEEE80211_ADDR_LEN];
+	memset(allsta, 0xff, IEEE80211_ADDR_LEN);
+	return madwifi_sta_deauth(priv, allsta, IEEE80211_REASON_AUTH_LEAVE);
+#else /* MADWIFI_BSD */
 	return 0;		/* XXX */
-#endif
+#endif /* MADWIFI_BSD */
 }
 
 
@@ -486,17 +492,51 @@ madwifi_read_sta_driver_data(void *priv, struct hostap_sta_driver_data *data,
 					u8 *addr)
 {
 	struct madwifi_driver_data *drv = priv;
+
+#ifdef MADWIFI_BSD
+	struct ieee80211req_sta_stats stats;
+
+	memset(data, 0, sizeof(*data));
+
+	/*
+	 * Fetch statistics for station from the system.
+	 */
+	memset(&stats, 0, sizeof(stats));
+	memcpy(stats.is_u.macaddr, addr, IEEE80211_ADDR_LEN);
+	if (set80211priv(drv, IEEE80211_IOCTL_GETSTASTATS, &stats,
+			 sizeof(stats))) {
+		if (memcmp(addr, drv->acct_mac, ETH_ALEN) == 0) {
+			memcpy(data, &drv->acct_data, sizeof(*data));
+			return 0;
+		}
+
+		printf("Failed to get station stats information element.\n");
+		return -1;
+	}
+
+	data->rx_packets = stats.is_stats.ns_rx_data;
+	data->rx_bytes = stats.is_stats.ns_rx_bytes;
+	data->tx_packets = stats.is_stats.ns_tx_data;
+	data->tx_bytes = stats.is_stats.ns_tx_bytes;
+	return 0;
+
+#else /* MADWIFI_BSD */
+
 	char buf[1024], line[128], *pos;
 	FILE *f;
 	unsigned long val;
 
 	memset(data, 0, sizeof(*data));
-	snprintf(buf, sizeof(buf), "/proc/net/hostap/%s/" MACSTR,
+	snprintf(buf, sizeof(buf), "/proc/net/madwifi/%s/" MACSTR,
 		 drv->iface, MAC2STR(addr));
 
 	f = fopen(buf, "r");
-	if (!f)
-		return -1;
+	if (!f) {
+		if (memcmp(addr, drv->acct_mac, ETH_ALEN) != 0)
+			return -1;
+		memcpy(data, &drv->acct_data, sizeof(*data));
+		return 0;
+	}
 	/* Need to read proc file with in one piece, so use large enough
 	 * buffer. */
 	setbuffer(f, buf, sizeof(buf));
@@ -520,8 +560,31 @@ madwifi_read_sta_driver_data(void *priv, struct hostap_sta_driver_data *data,
 	fclose(f);
 
 	return 0;
+#endif /* MADWIFI_BSD */
 }
 
+
+static int
+madwifi_sta_clear_stats(void *priv, u8 *addr)
+{
+#ifdef MADWIFI_BSD
+	struct madwifi_driver_data *drv = priv;
+	struct hostapd_data *hapd = drv->hapd;
+	struct ieee80211req_mlme mlme;
+	
+	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL, "%s: addr=%s\n",
+		      __func__, ether_sprintf(addr));
+
+	mlme.im_op = IEEE80211_MLME_CLEAR_STATS;
+	memcpy(mlme.im_macaddr, addr, IEEE80211_ADDR_LEN);
+	return set80211priv(priv, IEEE80211_IOCTL_SETMLME, &mlme,
+			    sizeof(mlme));
+#else /* MADWIFI_BSD */
+	return 0; /* FIX */
+#endif /* MADWIFI_BSD */
+}
+
+
 static int
 madwifi_set_opt_ie(void *priv, const u8 *ie, size_t ie_len)
 {
@@ -643,6 +706,12 @@ madwifi_new_sta(struct madwifi_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 		if (sta == NULL)
 			return -1;
 	}
+
+	if (memcmp(addr, drv->acct_mac, ETH_ALEN) == 0) {
+		/* Cached accounting data is not valid anymore. */
+		memset(drv->acct_mac, 0, ETH_ALEN);
+		memset(&drv->acct_data, 0, sizeof(drv->acct_data));
+	}
 	accounting_sta_get_id(hapd, sta);
 
 	if (hapd->conf->wpa) {
@@ -696,6 +765,29 @@ madwifi_wireless_event_wireless_custom(struct madwifi_driver_data *drv,
 				      "MLME-MICHAELMICFAILURE.indication "
 				      "with invalid MAC address");
 		}
+	} else if (strncmp(custom, "STA-TRAFFIC-STAT", 16) == 0) {
+		char *key, *value;
+		u32 val;
+		key = custom;
+		while ((key = strchr(key, '\n')) != NULL) {
+			key++;
+			value = strchr(key, '=');
+			if (value == NULL)
+				continue;
+			*value++ = '\0';
+			val = strtoul(value, NULL, 10);
+			if (strcmp(key, "mac") == 0)
+				hwaddr_aton(value, drv->acct_mac);
+			else if (strcmp(key, "rx_packets") == 0)
+				drv->acct_data.rx_packets = val;
+			else if (strcmp(key, "tx_packets") == 0)
+				drv->acct_data.tx_packets = val;
+			else if (strcmp(key, "rx_bytes") == 0)
+				drv->acct_data.rx_bytes = val;
+			else if (strcmp(key, "tx_bytes") == 0)
+				drv->acct_data.tx_bytes = val;
+			key = value;
+		}
 	}
 }
 
@@ -936,6 +1028,7 @@ madwifi_init(struct hostapd_data *hapd)
 {
 	struct madwifi_driver_data *drv;
 	struct ifreq ifr;
+	struct iwreq iwr;
 
 	drv = malloc(sizeof(struct madwifi_driver_data));
 	if (drv == NULL) {
@@ -978,6 +1071,17 @@ madwifi_init(struct hostapd_data *hapd)
 	} else
 		drv->sock_recv = drv->sock_xmit;
 
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
+
+	iwr.u.mode = IW_MODE_MASTER;
+
+	if (ioctl(drv->ioctl_sock, SIOCSIWMODE, &iwr) < 0) {
+		perror("ioctl[SIOCSIWMODE]");
+		printf("Could not set interface to master mode!\n");
+		goto bad;
+	}
+
 	madwifi_set_iface_flags(drv, 0);	/* mark down during setup */
 
 	hapd->driver = &drv->ops;
@@ -1070,6 +1174,7 @@ static const struct driver_ops madwifi_driver_ops = {
 	.sta_deauth		= madwifi_sta_deauth,
 	.set_ssid		= madwifi_set_ssid,
 	.get_ssid		= madwifi_get_ssid,
+	.sta_clear_stats        = madwifi_sta_clear_stats,
 };
 
 void madwifi_driver_register(void)
