@@ -703,7 +703,7 @@ eapol_sm_alloc(hostapd *hapd, struct sta_info *sta)
 	else
 		sm->portValid = TRUE;
 
-	if (hapd->conf->eap_authenticator) {
+	if (hapd->conf->eap_server) {
 		struct eap_config eap_conf;
 		memset(&eap_conf, 0, sizeof(eap_conf));
 		eap_conf.ssl_ctx = hapd->ssl_ctx;
@@ -1181,6 +1181,14 @@ static int eapol_sm_get_eap_user(void *ctx, const u8 *identity,
 }
 
 
+static const char * eapol_sm_get_eap_req_id_text(void *ctx, size_t *len)
+{
+	struct eapol_state_machine *sm = ctx;
+	*len = sm->hapd->conf->eap_req_id_text_len;
+	return sm->hapd->conf->eap_req_id_text;
+}
+
+
 static struct eapol_callbacks eapol_cb =
 {
 	.get_bool = eapol_sm_get_bool,
@@ -1188,4 +1196,5 @@ static struct eapol_callbacks eapol_cb =
 	.set_eapReqData = eapol_sm_set_eapReqData,
 	.set_eapKeyData = eapol_sm_set_eapKeyData,
 	.get_eap_user = eapol_sm_get_eap_user,
+	.get_eap_req_id_text = eapol_sm_get_eap_req_id_text,
 };
