@@ -24,6 +24,7 @@
 #include <stdarg.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <arpa/inet.h>
 
 #include "eloop.h"
 #include "hostapd.h"
@@ -58,8 +59,8 @@ extern int wpa_debug_show_keys;
 extern int wpa_debug_timestamp;
 
 
-void hostapd_logger(hostapd *hapd, u8 *addr, unsigned int module, int level,
-		    char *fmt, ...)
+void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
+		    unsigned int module, int level, const char *fmt, ...)
 {
 	char *format, *module_str;
 	int maxlen;
@@ -158,6 +159,28 @@ void hostapd_logger(hostapd *hapd, u8 *addr, unsigned int module, int level,
 }
 
 
+const char * hostapd_ip_txt(const struct hostapd_ip_addr *addr, char *buf,
+			    size_t buflen)
+{
+	if (buflen == 0 || addr == NULL)
+		return NULL;
+
+	if (addr->af == AF_INET) {
+		snprintf(buf, buflen, "%s", inet_ntoa(addr->u.v4));
+	} else {
+		buf[0] = '\0';
+	}
+#ifdef CONFIG_IPV6
+	if (addr->af == AF_INET6) {
+		if (inet_ntop(AF_INET6, &addr->u.v6, buf, buflen) == NULL)
+			buf[0] = '\0';
+	}
+#endif /* CONFIG_IPV6 */
+
+	return buf;
+}
+
+
 static void hostapd_deauth_all_stas(hostapd *hapd)
 {
 #if 0
@@ -186,7 +209,7 @@ void hostapd_new_assoc_sta(hostapd *hapd, struct sta_info *sta)
 	if (!hapd->conf->ieee802_1x)
 		accounting_sta_start(hapd, sta);
 
-	/* Start IEEE 802.1x authentication process for new stations */
+	/* Start IEEE 802.1X authentication process for new stations */
 	ieee802_1x_new_station(hapd, sta);
 	wpa_new_station(hapd, sta);
 }
@@ -438,7 +461,9 @@ static int hostapd_setup_interface(struct hostapd_data *hapd)
 		return -1;
 	}
 
-	hapd->radius = radius_client_init(hapd);
+	if (HOSTAPD_DEBUG_COND(HOSTAPD_DEBUG_MSGDUMPS))
+		conf->radius->msg_dumps = 1;
+	hapd->radius = radius_client_init(hapd, conf->radius);
 	if (hapd->radius == NULL) {
 		printf("RADIUS client initialization failed.\n");
 		return -1;
@@ -451,6 +476,7 @@ static int hostapd_setup_interface(struct hostapd_data *hapd)
 		srv.hostapd_conf = conf;
 		srv.eap_sim_db_priv = hapd->eap_sim_db_priv;
 		srv.ssl_ctx = hapd->ssl_ctx;
+		srv.ipv6 = conf->radius_server_ipv6;
 		hapd->radius_srv = radius_server_init(&srv);
 		if (hapd->radius_srv == NULL) {
 			printf("RADIUS server initialization failed.\n");
@@ -580,8 +606,8 @@ static void show_version(void)
 {
 	fprintf(stderr,
 		"hostapd v" VERSION_STR "\n"
-		"Host AP user space daemon for management functionality of "
-		"Host AP kernel driver\n"
+		"User space daemon for IEEE 802.11 AP management,\n"
+		"IEEE 802.1X/WPA/WPA2/EAP/RADIUS Authenticator\n"
 		"Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi> "
 		"and contributors\n");
 }
@@ -592,7 +618,7 @@ static void usage(void)
 	show_version();
 	fprintf(stderr,
 		"\n"
-		"usage: hostapd [-hdB] <configuration file(s)>\n"
+		"usage: hostapd [-hdBKt] <configuration file(s)>\n"
 		"\n"
 		"options:\n"
 		"   -h   show this usage\n"
@@ -634,9 +660,9 @@ static hostapd * hostapd_init(const char *config_file)
 	}
 
 #ifdef EAP_TLS_FUNCS
-	if (hapd->conf->eap_authenticator &&
+	if (hapd->conf->eap_server &&
 	    (hapd->conf->ca_cert || hapd->conf->server_cert)) {
-		hapd->ssl_ctx = tls_init();
+		hapd->ssl_ctx = tls_init(NULL);
 		if (hapd->ssl_ctx == NULL) {
 			printf("Failed to initialize TLS\n");
 			goto fail;
@@ -657,6 +683,12 @@ static hostapd * hostapd_init(const char *config_file)
 					   hapd->conf->private_key_passwd)) {
 			printf("Failed to load private key (%s)\n",
 			       hapd->conf->private_key);
+			goto fail;
+		}
+		if (tls_global_set_verify(hapd->ssl_ctx,
+					  hapd->conf->check_crl)) {
+			printf("Failed to enable check_crl\n");
+			goto fail;
 		}
 	}
 #endif /* EAP_TLS_FUNCS */
