@@ -175,7 +175,7 @@ ParseRes ieee802_11_parse_elems(struct hostapd_data *hapd, u8 *start,
 }
 
 
-static void ieee802_11_print_ssid(u8 *ssid, u8 len)
+static void ieee802_11_print_ssid(const u8 *ssid, u8 len)
 {
 	int i;
 	for (i = 0; i < len; i++) {
@@ -199,7 +199,8 @@ static void ieee802_11_sta_authenticate(void *eloop_ctx, void *timeout_ctx)
 
 	printf("Authenticate with AP " MACSTR " SSID=",
 	       MAC2STR(hapd->conf->assoc_ap_addr));
-	ieee802_11_print_ssid(hapd->assoc_ap_ssid, hapd->assoc_ap_ssid_len);
+	ieee802_11_print_ssid((u8 *) hapd->assoc_ap_ssid,
+			      hapd->assoc_ap_ssid_len);
 	printf(" (as station)\n");
 
 	memset(&mgmt, 0, sizeof(mgmt));
@@ -236,7 +237,8 @@ static void ieee802_11_sta_associate(void *eloop_ctx, void *timeout_ctx)
 
 	printf("Associate with AP " MACSTR " SSID=",
 	       MAC2STR(hapd->conf->assoc_ap_addr));
-	ieee802_11_print_ssid(hapd->assoc_ap_ssid, hapd->assoc_ap_ssid_len);
+	ieee802_11_print_ssid((u8 *) hapd->assoc_ap_ssid,
+			      hapd->assoc_ap_ssid_len);
 	printf(" (as station)\n");
 
 	memset(mgmt, 0, sizeof(*mgmt));
@@ -475,7 +477,7 @@ static void handle_auth(hostapd *hapd, struct ieee80211_mgmt *mgmt, size_t len)
 	sta->flags &= ~WLAN_STA_PREAUTH;
 	ieee802_1x_notify_pre_auth(sta->eapol_sm, 0);
 
-	if (hapd->conf->radius_acct_interim_interval == 0 &&
+	if (hapd->conf->radius->acct_interim_interval == 0 &&
 	    acct_interim_interval)
 		sta->acct_interim_interval = acct_interim_interval;
 	if (res == HOSTAPD_ACL_ACCEPT_TIMEOUT)
