@@ -30,5 +30,8 @@ void l2_packet_deinit(struct l2_packet_data *l2);
 int l2_packet_get_own_addr(struct l2_packet_data *l2, u8 *addr);
 int l2_packet_send(struct l2_packet_data *l2, u8 *buf, size_t len);
 void l2_packet_set_rx_l2_hdr(struct l2_packet_data *l2, int rx_l2_hdr);
+void * l2_packet_set_ethhdr(struct l2_ethhdr *ethhdr, const u8 *dest,
+			    const u8 *source, u16 proto);
+int l2_packet_get_ip_addr(struct l2_packet_data *l2, char *buf, size_t len);
 
 #endif /* L2_PACKET_H */
