@@ -1,12 +1,16 @@
 /*
  * Host AP (software wireless LAN access point) user space daemon for
  * Host AP kernel driver / RADIUS client
- * Copyright (c) 2002-2003, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
- * published by the Free Software Foundation. See README and COPYING for
- * more details.
+ * published by the Free Software Foundation.
+ *
+ * Alternatively, this software may be distributed under the terms of BSD
+ * license.
+ *
+ * See README and COPYING for more details.
  */
 
 #include <stdlib.h>
@@ -120,8 +124,10 @@ static const char *radius_code_string(u8 code)
 struct radius_attr_type {
 	u8 type;
 	char *name;
-	enum { RADIUS_ATTR_UNDIST, RADIUS_ATTR_TEXT, RADIUS_ATTR_IP,
-	       RADIUS_ATTR_HEXDUMP, RADIUS_ATTR_INT32 } data_type;
+	enum {
+		RADIUS_ATTR_UNDIST, RADIUS_ATTR_TEXT, RADIUS_ATTR_IP,
+		RADIUS_ATTR_HEXDUMP, RADIUS_ATTR_INT32, RADIUS_ATTR_IPV6
+	} data_type;
 };
 
 static struct radius_attr_type radius_attrs[] =
@@ -163,14 +169,20 @@ static struct radius_attr_type radius_attrs[] =
 	{ RADIUS_ATTR_ACCT_MULTI_SESSION_ID, "Acct-Multi-Session-Id",
 	  RADIUS_ATTR_TEXT },
 	{ RADIUS_ATTR_ACCT_LINK_COUNT, "Acct-Link-Count", RADIUS_ATTR_INT32 },
+	{ RADIUS_ATTR_ACCT_INPUT_GIGAWORDS, "Acct-Input-Gigawords", 
+	  RADIUS_ATTR_INT32 },
+	{ RADIUS_ATTR_ACCT_OUTPUT_GIGAWORDS, "Acct-Output-Gigawords",
+	  RADIUS_ATTR_INT32 },
+	{ RADIUS_ATTR_EVENT_TIMESTAMP, "Event-Timestamp",
+	  RADIUS_ATTR_INT32 },
 	{ RADIUS_ATTR_NAS_PORT_TYPE, "NAS-Port-Type", RADIUS_ATTR_INT32 },
 	{ RADIUS_ATTR_CONNECT_INFO, "Connect-Info", RADIUS_ATTR_TEXT },
 	{ RADIUS_ATTR_EAP_MESSAGE, "EAP-Message", RADIUS_ATTR_UNDIST },
 	{ RADIUS_ATTR_MESSAGE_AUTHENTICATOR, "Message-Authenticator",
 	  RADIUS_ATTR_UNDIST },
 	{ RADIUS_ATTR_ACCT_INTERIM_INTERVAL, "Acct-Interim-Interval",
-	  RADIUS_ATTR_INT32 }
-
+	  RADIUS_ATTR_INT32 },
+	{ RADIUS_ATTR_NAS_IPV6_ADDRESS, "NAS-IPv6-Address", RADIUS_ATTR_IPV6 },
 };
 #define RADIUS_ATTRS (sizeof(radius_attrs) / sizeof(radius_attrs[0]))
 
@@ -192,7 +204,7 @@ static void radius_msg_dump_attr(struct radius_attr_hdr *hdr)
 {
 	struct radius_attr_type *attr;
 	int i, len;
-	char *pos;
+	unsigned char *pos;
 
 	attr = radius_get_attr_type(hdr->type);
 
@@ -203,7 +215,7 @@ static void radius_msg_dump_attr(struct radius_attr_hdr *hdr)
 		return;
 
 	len = hdr->length - sizeof(struct radius_attr_hdr);
-	pos = (char *) (hdr + 1);
+	pos = (unsigned char *) (hdr + 1);
 
 	switch (attr->data_type) {
 	case RADIUS_ATTR_TEXT:
@@ -221,7 +233,21 @@ static void radius_msg_dump_attr(struct radius_attr_hdr *hdr)
 			printf("      Invalid IP address length %d\n", len);
 		break;
 
+#ifdef CONFIG_IPV6
+	case RADIUS_ATTR_IPV6:
+		if (len == 16) {
+			char buf[128];
+			const char *atxt;
+			struct in6_addr *addr = (struct in6_addr *) pos;
+			atxt = inet_ntop(AF_INET6, addr, buf, sizeof(buf));
+			printf("      Value: %s\n", atxt ? atxt : "?");
+		} else
+			printf("      Invalid IPv6 address length %d\n", len);
+		break;
+#endif /* CONFIG_IPV6 */
+
 	case RADIUS_ATTR_HEXDUMP:
+	case RADIUS_ATTR_UNDIST:
 		printf("      Value:");
 		for (i = 0; i < len; i++)
 			printf(" %02x", pos[i]);
@@ -256,16 +282,21 @@ void radius_msg_dump(struct radius_msg *msg)
 }
 
 
-void radius_msg_finish(struct radius_msg *msg, u8 *secret, size_t secret_len)
+int radius_msg_finish(struct radius_msg *msg, u8 *secret, size_t secret_len)
 {
 	if (secret) {
-		char auth[MD5_MAC_LEN];
+		u8 auth[MD5_MAC_LEN];
 		struct radius_attr_hdr *attr;
 
 		memset(auth, 0, MD5_MAC_LEN);
 		attr = radius_msg_add_attr(msg,
 					   RADIUS_ATTR_MESSAGE_AUTHENTICATOR,
 					   auth, MD5_MAC_LEN);
+		if (attr == NULL) {
+			printf("WARNING: Could not add "
+			       "Message-Authenticator\n");
+			return -1;
+		}
 		msg->hdr->length = htons(msg->buf_used);
 		hmac_md5(secret, secret_len, msg->buf, msg->buf_used,
 			 (u8 *) (attr + 1));
@@ -273,9 +304,49 @@ void radius_msg_finish(struct radius_msg *msg, u8 *secret, size_t secret_len)
 		msg->hdr->length = htons(msg->buf_used);
 
 	if (msg->buf_used > 0xffff) {
-		printf("WARNING: too long RADIUS messages (%d)\n",
-		       msg->buf_used);
+		printf("WARNING: too long RADIUS message (%lu)\n",
+		       (unsigned long) msg->buf_used);
+		return -1;
 	}
+	return 0;
+}
+
+
+int radius_msg_finish_srv(struct radius_msg *msg, const u8 *secret,
+			  size_t secret_len, const u8 *req_authenticator)
+{
+	u8 auth[MD5_MAC_LEN];
+	struct radius_attr_hdr *attr;
+	MD5_CTX context;
+
+	memset(auth, 0, MD5_MAC_LEN);
+	attr = radius_msg_add_attr(msg, RADIUS_ATTR_MESSAGE_AUTHENTICATOR,
+				   auth, MD5_MAC_LEN);
+	if (attr == NULL) {
+		printf("WARNING: Could not add Message-Authenticator\n");
+		return -1;
+	}
+	msg->hdr->length = htons(msg->buf_used);
+	memcpy(msg->hdr->authenticator, req_authenticator,
+	       sizeof(msg->hdr->authenticator));
+	hmac_md5(secret, secret_len, msg->buf, msg->buf_used,
+		 (u8 *) (attr + 1));
+
+	/* ResponseAuth = MD5(Code+ID+Length+RequestAuth+Attributes+Secret) */
+	MD5Init(&context);
+	MD5Update(&context, (u8 *) msg->hdr, 1 + 1 + 2);
+	MD5Update(&context, req_authenticator, MD5_MAC_LEN);
+	MD5Update(&context, (u8 *) (msg->hdr + 1),
+		  msg->buf_used - sizeof(*msg->hdr));
+	MD5Update(&context, secret, secret_len);
+	MD5Final(msg->hdr->authenticator, &context);
+
+	if (msg->buf_used > 0xffff) {
+		printf("WARNING: too long RADIUS message (%lu)\n",
+		       (unsigned long) msg->buf_used);
+		return -1;
+	}
+	return 0;
 }
 
 
@@ -292,8 +363,8 @@ void radius_msg_finish_acct(struct radius_msg *msg, u8 *secret,
 	MD5Final(msg->hdr->authenticator, &context);
 
 	if (msg->buf_used > 0xffff) {
-		printf("WARNING: too long RADIUS messages (%d)\n",
-		       msg->buf_used);
+		printf("WARNING: too long RADIUS messages (%lu)\n",
+		       (unsigned long) msg->buf_used);
 	}
 }
 
@@ -328,8 +399,8 @@ struct radius_attr_hdr *radius_msg_add_attr(struct radius_msg *msg, u8 type,
 	struct radius_attr_hdr *attr;
 
 	if (data_len > RADIUS_MAX_ATTR_LEN) {
-		printf("radius_msg_add_attr: too long attribute (%d bytes)\n",
-		       data_len);
+		printf("radius_msg_add_attr: too long attribute (%lu bytes)\n",
+		       (unsigned long) data_len);
 		return NULL;
 	}
 
@@ -392,8 +463,8 @@ struct radius_msg *radius_msg_parse(const u8 *data, size_t len)
 	}
 
 	if (msg_len < len) {
-		printf("Ignored %d extra bytes after RADIUS message\n",
-		       len - msg_len);
+		printf("Ignored %lu extra bytes after RADIUS message\n",
+		       (unsigned long) len - msg_len);
 	}
 
 	msg = (struct radius_msg *) malloc(sizeof(*msg));
@@ -461,9 +532,9 @@ int radius_msg_add_eap(struct radius_msg *msg, u8 *data, size_t data_len)
 }
 
 
-char *radius_msg_get_eap(struct radius_msg *msg, size_t *eap_len)
+u8 *radius_msg_get_eap(struct radius_msg *msg, size_t *eap_len)
 {
-	char *eap, *pos;
+	u8 *eap, *pos;
 	size_t len;
 	int i;
 
@@ -480,7 +551,7 @@ char *radius_msg_get_eap(struct radius_msg *msg, size_t *eap_len)
 	if (len == 0)
 		return NULL;
 
-	eap = (char *) malloc(len);
+	eap = malloc(len);
 	if (eap == NULL)
 		return NULL;
 
@@ -501,24 +572,17 @@ char *radius_msg_get_eap(struct radius_msg *msg, size_t *eap_len)
 }
 
 
-int radius_msg_verify(struct radius_msg *msg, u8 *secret, size_t secret_len,
-		      struct radius_msg *sent_msg)
+int radius_msg_verify_msg_auth(struct radius_msg *msg, const u8 *secret,
+			       size_t secret_len, const u8 *req_auth)
 {
 	u8 auth[MD5_MAC_LEN], orig[MD5_MAC_LEN];
 	u8 orig_authenticator[16];
 	struct radius_attr_hdr *attr = NULL;
 	int i;
-	MD5_CTX context;
-	u8 hash[MD5_MAC_LEN];
-
-	if (sent_msg == NULL) {
-		printf("No matching Access-Request message found\n");
-		return 1;
-	}
 
 	for (i = 0; i < msg->attr_used; i++) {
 		if (msg->attrs[i]->type == RADIUS_ATTR_MESSAGE_AUTHENTICATOR) {
-			if (attr != 0) {
+			if (attr != NULL) {
 				printf("Multiple Message-Authenticator "
 				       "attributes in RADIUS message\n");
 				return 1;
@@ -534,20 +598,44 @@ int radius_msg_verify(struct radius_msg *msg, u8 *secret, size_t secret_len,
 
 	memcpy(orig, attr + 1, MD5_MAC_LEN);
 	memset(attr + 1, 0, MD5_MAC_LEN);
-	memcpy(orig_authenticator, msg->hdr->authenticator,
-	       sizeof(orig_authenticator));
-	memcpy(msg->hdr->authenticator, sent_msg->hdr->authenticator,
-	       sizeof(msg->hdr->authenticator));
+	if (req_auth) {
+		memcpy(orig_authenticator, msg->hdr->authenticator,
+		       sizeof(orig_authenticator));
+		memcpy(msg->hdr->authenticator, req_auth,
+		       sizeof(msg->hdr->authenticator));
+	}
 	hmac_md5(secret, secret_len, msg->buf, msg->buf_used, auth);
 	memcpy(attr + 1, orig, MD5_MAC_LEN);
-	memcpy(msg->hdr->authenticator, orig_authenticator,
-	       sizeof(orig_authenticator));
+	if (req_auth) {
+		memcpy(msg->hdr->authenticator, orig_authenticator,
+		       sizeof(orig_authenticator));
+	}
 
 	if (memcmp(orig, auth, MD5_MAC_LEN) != 0) {
 		printf("Invalid Message-Authenticator!\n");
 		return 1;
 	}
 
+	return 0;
+}
+
+
+int radius_msg_verify(struct radius_msg *msg, u8 *secret, size_t secret_len,
+		      struct radius_msg *sent_msg)
+{
+	MD5_CTX context;
+	u8 hash[MD5_MAC_LEN];
+
+	if (sent_msg == NULL) {
+		printf("No matching Access-Request message found\n");
+		return 1;
+	}
+
+	if (radius_msg_verify_msg_auth(msg, secret, secret_len,
+				       sent_msg->hdr->authenticator)) {
+		return 1;
+	}
+
 	/* ResponseAuth = MD5(Code+ID+Length+RequestAuth+Attributes+Secret) */
 	MD5Init(&context);
 	MD5Update(&context, (u8 *) msg->hdr, 1 + 1 + 2);
@@ -642,7 +730,7 @@ void radius_msg_make_authenticator(struct radius_msg *msg,
 static u8 *radius_msg_get_vendor_attr(struct radius_msg *msg, u32 vendor,
 				      u8 subtype, size_t *alen)
 {
-	char *data, *pos;
+	u8 *data, *pos;
 	int i;
 	size_t len;
 
@@ -653,7 +741,7 @@ static u8 *radius_msg_get_vendor_attr(struct radius_msg *msg, u32 vendor,
 		struct radius_attr_hdr *attr = msg->attrs[i];
 		int left;
 		u32 vendor_id;
-		struct radius_attr_vendor_microsoft *ms;
+		struct radius_attr_vendor *vhdr;
 
 		if (attr->type != RADIUS_ATTR_VENDOR_SPECIFIC)
 			continue;
@@ -662,7 +750,7 @@ static u8 *radius_msg_get_vendor_attr(struct radius_msg *msg, u32 vendor,
 		if (left < 4)
 			continue;
 
-		pos = (char *) (attr + 1);
+		pos = (u8 *) (attr + 1);
 
 		memcpy(&vendor_id, pos, 4);
 		pos += 4;
@@ -671,23 +759,24 @@ static u8 *radius_msg_get_vendor_attr(struct radius_msg *msg, u32 vendor,
 		if (ntohl(vendor_id) != vendor)
 			continue;
 
-		while (left >= sizeof(*ms)) {
-			ms = (struct radius_attr_vendor_microsoft *) pos;
-			if (ms->vendor_length > left) {
+		while (left >= sizeof(*vhdr)) {
+			vhdr = (struct radius_attr_vendor *) pos;
+			if (vhdr->vendor_length > left ||
+			    vhdr->vendor_length < sizeof(*vhdr)) {
 				left = 0;
-				continue;
+				break;
 			}
-			if (ms->vendor_type != subtype) {
-				pos += ms->vendor_length;
-				left -= ms->vendor_length;
+			if (vhdr->vendor_type != subtype) {
+				pos += vhdr->vendor_length;
+				left -= vhdr->vendor_length;
 				continue;
 			}
 
-			len = ms->vendor_length - sizeof(*ms);
-			data = (char *) malloc(len);
+			len = vhdr->vendor_length - sizeof(*vhdr);
+			data = malloc(len);
 			if (data == NULL)
 				return NULL;
-			memcpy(data, pos + sizeof(*ms), len);
+			memcpy(data, pos + sizeof(*vhdr), len);
 			if (alen)
 				*alen = len;
 			return data;
@@ -698,10 +787,12 @@ static u8 *radius_msg_get_vendor_attr(struct radius_msg *msg, u32 vendor,
 }
 
 
-static u8 * decrypt_ms_key(u8 *key, size_t len, struct radius_msg *sent_msg,
-			   u8 *secret, size_t secret_len, size_t *reslen)
+static u8 * decrypt_ms_key(const u8 *key, size_t len,
+			   const u8 *req_authenticator,
+			   const u8 *secret, size_t secret_len, size_t *reslen)
 {
-	u8 *pos, *plain, *ppos, *res;
+	u8 *plain, *ppos, *res;
+	const u8 *pos;
 	size_t left, plen;
 	u8 hash[MD5_MAC_LEN];
 	MD5_CTX context;
@@ -715,7 +806,7 @@ static u8 * decrypt_ms_key(u8 *key, size_t len, struct radius_msg *sent_msg,
 	pos = key + 2;
 	left = len - 2;
 	if (left % 16) {
-		printf("Invalid ms key len\n");
+		printf("Invalid ms key len %lu\n", (unsigned long) left);
 		return NULL;
 	}
 
@@ -731,8 +822,7 @@ static u8 * decrypt_ms_key(u8 *key, size_t len, struct radius_msg *sent_msg,
 		MD5Init(&context);
 		MD5Update(&context, secret, secret_len);
 		if (first) {
-			MD5Update(&context, sent_msg->hdr->authenticator,
-				  MD5_MAC_LEN);
+			MD5Update(&context, req_authenticator, MD5_MAC_LEN);
 			MD5Update(&context, key, 2); /* Salt */
 			first = 0;
 		} else
@@ -763,6 +853,51 @@ static u8 * decrypt_ms_key(u8 *key, size_t len, struct radius_msg *sent_msg,
 }
 
 
+static void encrypt_ms_key(const u8 *key, size_t key_len, u16 salt,
+			   const u8 *req_authenticator,
+			   const u8 *secret, size_t secret_len,
+			   u8 *ebuf, size_t *elen)
+{
+	int i, len, first = 1;
+	u8 hash[MD5_MAC_LEN], saltbuf[2], *pos;
+	MD5_CTX context;
+
+	saltbuf[0] = salt >> 8;
+	saltbuf[1] = salt;
+
+	len = 1 + key_len;
+	if (len & 0x0f) {
+		len = (len & 0xf0) + 16;
+	}
+	memset(ebuf, 0, len);
+	ebuf[0] = key_len;
+	memcpy(ebuf + 1, key, key_len);
+
+	*elen = len;
+
+	pos = ebuf;
+	while (len > 0) {
+		/* b(1) = MD5(Secret + Request-Authenticator + Salt)
+		 * b(i) = MD5(Secret + c(i - 1)) for i > 1 */
+		MD5Init(&context);
+		MD5Update(&context, secret, secret_len);
+		if (first) {
+			MD5Update(&context, req_authenticator, MD5_MAC_LEN);
+			MD5Update(&context, saltbuf, sizeof(saltbuf));
+			first = 0;
+		} else {
+			MD5Update(&context, pos - MD5_MAC_LEN, MD5_MAC_LEN);
+		}
+		MD5Final(hash, &context);
+
+		for (i = 0; i < MD5_MAC_LEN; i++)
+			*pos++ ^= hash[i];
+
+		len -= MD5_MAC_LEN;
+	}
+}
+
+
 struct radius_ms_mppe_keys *
 radius_msg_get_ms_keys(struct radius_msg *msg, struct radius_msg *sent_msg,
 		       u8 *secret, size_t secret_len)
@@ -784,7 +919,8 @@ radius_msg_get_ms_keys(struct radius_msg *msg, struct radius_msg *sent_msg,
 					 RADIUS_VENDOR_ATTR_MS_MPPE_SEND_KEY,
 					 &keylen);
 	if (key) {
-		keys->send = decrypt_ms_key(key, keylen, sent_msg,
+		keys->send = decrypt_ms_key(key, keylen,
+					    sent_msg->hdr->authenticator,
 					    secret, secret_len,
 					    &keys->send_len);
 		free(key);
@@ -794,7 +930,8 @@ radius_msg_get_ms_keys(struct radius_msg *msg, struct radius_msg *sent_msg,
 					 RADIUS_VENDOR_ATTR_MS_MPPE_RECV_KEY,
 					 &keylen);
 	if (key) {
-		keys->recv = decrypt_ms_key(key, keylen, sent_msg,
+		keys->recv = decrypt_ms_key(key, keylen,
+					    sent_msg->hdr->authenticator,
 					    secret, secret_len,
 					    &keys->recv_len);
 		free(key);
@@ -824,7 +961,8 @@ radius_msg_get_cisco_keys(struct radius_msg *msg, struct radius_msg *sent_msg,
 	key = radius_msg_get_vendor_attr(msg, RADIUS_VENDOR_ID_CISCO,
 					 RADIUS_CISCO_AV_PAIR, &keylen);
 	if (key && keylen == 51 && memcmp(key, "leap:session-key=", 17) == 0) {
-		keys->recv = decrypt_ms_key(key + 17, keylen - 17, sent_msg,
+		keys->recv = decrypt_ms_key(key + 17, keylen - 17,
+					    sent_msg->hdr->authenticator,
 					    secret, secret_len,
 					    &keys->recv_len);
 		free(key);
@@ -834,6 +972,77 @@ radius_msg_get_cisco_keys(struct radius_msg *msg, struct radius_msg *sent_msg,
 }
 
 
+int radius_msg_add_mppe_keys(struct radius_msg *msg,
+			     const u8 *req_authenticator,
+			     const u8 *secret, size_t secret_len,
+			     const u8 *send_key, size_t send_key_len,
+			     const u8 *recv_key, size_t recv_key_len)
+{
+	struct radius_attr_hdr *attr;
+	u32 vendor_id = htonl(RADIUS_VENDOR_ID_MICROSOFT);
+	u8 *buf;
+	struct radius_attr_vendor *vhdr;
+	u8 *pos;
+	size_t elen;
+	int hlen;
+	u16 salt;
+
+	hlen = sizeof(vendor_id) + sizeof(*vhdr) + 2;
+
+	/* MS-MPPE-Send-Key */
+	buf = malloc(hlen + send_key_len + 16);
+	if (buf == NULL) {
+		return 0;
+	}
+	pos = buf;
+	memcpy(pos, &vendor_id, sizeof(vendor_id));
+	pos += sizeof(vendor_id);
+	vhdr = (struct radius_attr_vendor *) pos;
+	vhdr->vendor_type = RADIUS_VENDOR_ATTR_MS_MPPE_SEND_KEY;
+	pos = (u8 *) (vhdr + 1);
+	salt = random() | 0x8000;
+	*pos++ = salt >> 8;
+	*pos++ = salt;
+	encrypt_ms_key(send_key, send_key_len, salt, req_authenticator, secret,
+		       secret_len, pos, &elen);
+	vhdr->vendor_length = hlen + elen - sizeof(vendor_id);
+
+	attr = radius_msg_add_attr(msg, RADIUS_ATTR_VENDOR_SPECIFIC,
+				   buf, hlen + elen);
+	free(buf);
+	if (attr == NULL) {
+		return 0;
+	}
+
+	/* MS-MPPE-Recv-Key */
+	buf = malloc(hlen + send_key_len + 16);
+	if (buf == NULL) {
+		return 0;
+	}
+	pos = buf;
+	memcpy(pos, &vendor_id, sizeof(vendor_id));
+	pos += sizeof(vendor_id);
+	vhdr = (struct radius_attr_vendor *) pos;
+	vhdr->vendor_type = RADIUS_VENDOR_ATTR_MS_MPPE_RECV_KEY;
+	pos = (u8 *) (vhdr + 1);
+	salt ^= 1;
+	*pos++ = salt >> 8;
+	*pos++ = salt;
+	encrypt_ms_key(recv_key, recv_key_len, salt, req_authenticator, secret,
+		       secret_len, pos, &elen);
+	vhdr->vendor_length = hlen + elen - sizeof(vendor_id);
+
+	attr = radius_msg_add_attr(msg, RADIUS_ATTR_VENDOR_SPECIFIC,
+				   buf, hlen + elen);
+	free(buf);
+	if (attr == NULL) {
+		return 0;
+	}
+
+	return 1;
+}
+
+
 /* Add User-Password attribute to a RADIUS message and encrypt it as specified
  * in RFC 2865, Chap. 5.2 */
 struct radius_attr_hdr *
@@ -841,11 +1050,11 @@ radius_msg_add_attr_user_password(struct radius_msg *msg,
 				  u8 *data, size_t data_len,
 				  u8 *secret, size_t secret_len)
 {
-	char buf[128];
+	u8 buf[128];
 	int padlen, i, pos;
 	MD5_CTX context;
 	size_t buf_len;
-	char hash[16];
+	u8 hash[16];
 
 	if (data_len > 128)
 		return NULL;
@@ -910,13 +1119,14 @@ int radius_msg_get_attr(struct radius_msg *msg, u8 type, u8 *buf, size_t len)
 
 
 int radius_msg_get_attr_ptr(struct radius_msg *msg, u8 type, u8 **buf,
-			    size_t *len)
+			    size_t *len, const u8 *start)
 {
 	int i;
 	struct radius_attr_hdr *attr = NULL;
 
 	for (i = 0; i < msg->attr_used; i++) {
-		if (msg->attrs[i]->type == type) {
+		if (msg->attrs[i]->type == type &&
+		    (start == NULL || (u8 *) msg->attrs[i] > start)) {
 			attr = msg->attrs[i];
 			break;
 		}
@@ -929,3 +1139,18 @@ int radius_msg_get_attr_ptr(struct radius_msg *msg, u8 type, u8 **buf,
 	*len = attr->length - sizeof(*attr);
 	return 0;
 }
+
+
+int radius_msg_count_attr(struct radius_msg *msg, u8 type, int min_len)
+{
+	int i, count;
+
+	for (count = 0, i = 0; i < msg->attr_used; i++) {
+		if (msg->attrs[i]->type == type &&
+		    msg->attrs[i]->length >=
+		    sizeof(struct radius_attr_hdr) + min_len)
+			count++;
+	}
+
+	return count;
+}
