@@ -101,7 +101,7 @@ static const u8 RSN_KEY_DATA_PMKID[] = { 0x00, 0x0f, 0xac, 4 };
  * Authenticated Key Management Suite Count (2 octets, little endian)
  *    (default: 1)
  * Authenticated Key Management Suite List (4 * n octets)
- *    (default: unspec 802.1x)
+ *    (default: unspec 802.1X)
  * WPA Capabilities (2 octets, little endian) (default: 0)
  */
 
@@ -123,7 +123,7 @@ struct wpa_ie_hdr {
  * Authenticated Key Management Suite Count (2 octets, little endian)
  *    (default: 1)
  * Authenticated Key Management Suite List (4 * n octets)
- *    (default: unspec 802.1x)
+ *    (default: unspec 802.1X)
  * RSN Capabilities (2 octets, little endian) (default: 0)
  * PMKID Count (2 octets) (default: 0)
  * PMKID List (16 * n octets)
@@ -999,7 +999,7 @@ struct wpa_ie_data {
 };
 
 
-static int wpa_parse_wpa_ie_wpa(u8 *wpa_ie, size_t wpa_ie_len,
+static int wpa_parse_wpa_ie_wpa(const u8 *wpa_ie, size_t wpa_ie_len,
 				struct wpa_ie_data *data)
 {
 	struct wpa_ie_hdr *hdr;
@@ -1078,7 +1078,7 @@ static int wpa_parse_wpa_ie_wpa(u8 *wpa_ie, size_t wpa_ie_len,
 }
 
 
-static int wpa_parse_wpa_ie_rsn(u8 *rsn_ie, size_t rsn_ie_len,
+static int wpa_parse_wpa_ie_rsn(const u8 *rsn_ie, size_t rsn_ie_len,
 				struct wpa_ie_data *data)
 {
 	struct rsn_ie_hdr *hdr;
@@ -1172,7 +1172,7 @@ static int wpa_parse_wpa_ie_rsn(u8 *rsn_ie, size_t rsn_ie_len,
 
 
 int wpa_validate_wpa_ie(struct hostapd_data *hapd, struct sta_info *sta,
-			u8 *wpa_ie, size_t wpa_ie_len, int version)
+			const u8 *wpa_ie, size_t wpa_ie_len, int version)
 {
 	struct wpa_ie_data data;
 	int ciphers, key_mgmt, res, i;
